# frozen_string_literal: true

# Released under the MIT License.
# Copyright, 2019-2024, by Samuel Williams.

module Falcon
	# Pre-defined environments for hosting web applications.
	#
	# See {Configuration::Loader#load} for more details.
	module Environment
		LEGACY_ENVIRONMENTS = {}
	end
end
